@@ -32,8 +32,8 @@
  * D3 (5)	  Y3
  * D4 (6)	  Y4
  * D5 (7)	  Y5
- * D6 (7)	  Y6
- * D7 (7)	  Y7
+ * D6 (8)	  Y6
+ * D7 (9)	  Y7
  * nSTRB  (1)     Y8
  * nSEL   (17)    Y9
  * nACK   (10)    X0
@@ -52,7 +52,7 @@
  */
 
 #include "hd44780-ext8bit.h"
-#include "hd44780.h"
+#include "hd44780-low.h"
 #include "lpt-port.h"
 
 #include "port.h"
@@ -67,57 +67,57 @@
 // HD44780_senddata
 // HD44780_readkeypad
 
-void lcdtime_HD44780_senddata (unsigned char displayID, unsigned char flags, unsigned char ch);
-void lcdtime_HD44780_backlight (unsigned char state);
-unsigned char lcdtime_HD44780_readkeypad (unsigned int YData);
+void lcdtime_HD44780_senddata (PrivateData *p, unsigned char displayID, unsigned char flags, unsigned char ch);
+void lcdtime_HD44780_backlight (PrivateData *p, unsigned char state);
+unsigned char lcdtime_HD44780_readkeypad (PrivateData *p, unsigned int YData);
+void lcdtime_HD44780_output(PrivateData *p, int data);
 
 #define RS	STRB
 #define RW	LF
 #define EN1	INIT
 #define BL	SEL
-
-static unsigned int lptPort;
-static char stuckinputs = 0;	// if an input line is stuck, it will be ignored
-static char backlight_bit = 0;
+#define LE	SEL
 
 static int semid;
 
 // initialise the driver
 int
-hd_init_ext8bit (HD44780_functions * hd44780_functions, lcd_logical_driver * driver, char *args, unsigned int port)
+hd_init_ext8bit (Driver *drvthis)
 {
+	PrivateData *p = (PrivateData*) drvthis->private_data;
+	HD44780_functions *hd44780_functions = p->hd44780_functions;
+
 	semid = sem_get ();
 
 	// Reserve the port registers
-	lptPort = port;
-	port_access(lptPort);
-	port_access(lptPort+1);
-	port_access(lptPort+2);
+	port_access_multiple(p->port,3);
 
 	hd44780_functions->senddata = lcdtime_HD44780_senddata;
 	hd44780_functions->backlight = lcdtime_HD44780_backlight;
 	hd44780_functions->readkeypad = lcdtime_HD44780_readkeypad;
 
 	// setup the lcd in 8 bit mode
-	hd44780_functions->senddata (0, RS_INSTR, FUNCSET | IF_8BIT);
-	hd44780_functions->uPause (4100);
-	hd44780_functions->senddata (0, RS_INSTR, FUNCSET | IF_8BIT);
-	hd44780_functions->uPause (100);
-	hd44780_functions->senddata (0, RS_INSTR, FUNCSET | IF_8BIT | TWOLINE | SMALLCHAR);
-	hd44780_functions->uPause (40);
+	hd44780_functions->senddata (p, 0, RS_INSTR, FUNCSET | IF_8BIT);
+	hd44780_functions->uPause (p, 4100);
+	hd44780_functions->senddata (p, 0, RS_INSTR, FUNCSET | IF_8BIT);
+	hd44780_functions->uPause (p, 100);
+	hd44780_functions->senddata (p, 0, RS_INSTR, FUNCSET | IF_8BIT | TWOLINE | SMALLCHAR);
+	hd44780_functions->uPause (p, 40);
 
-	common_init ();
+	common_init (p, IF_8BIT);
 
-	if (have_keypad) {
+	if (p->have_keypad) {
 		// Remember which input lines are stuck
-		stuckinputs = lcdtime_HD44780_readkeypad (0);
+		p->stuckinputs = lcdtime_HD44780_readkeypad (p, 0);
 	}
+	// Writes new value to the "bargraph" latches
+	hd44780_functions->output = lcdtime_HD44780_output;
 	return 0;
 }
 
 // lcdtime_HD44780_senddata
 void
-lcdtime_HD44780_senddata (unsigned char displayID, unsigned char flags, unsigned char ch)
+lcdtime_HD44780_senddata (PrivateData *p, unsigned char displayID, unsigned char flags, unsigned char ch)
 {
 	unsigned char enableLines = 0, portControl;
 
@@ -129,46 +129,47 @@ lcdtime_HD44780_senddata (unsigned char displayID, unsigned char flags, unsigned
 	else //if (iflags == RS_DATA)
 		portControl = RS;
 
-	portControl |= backlight_bit;
+	portControl |= p->backlight_bit;
 
 	sem_wait (semid);
-	port_out (lptPort + 2, portControl ^ OUTMASK);
-	port_out (lptPort, ch);
-	if( delayBus ) hd44780_functions->uPause (1);
-	port_out (lptPort + 2, (enableLines|portControl) ^ OUTMASK);
-	if( delayBus ) hd44780_functions->uPause (1);
-	port_out (lptPort + 2, portControl ^ OUTMASK);
+	port_out (p->port + 2, portControl ^ OUTMASK);
+	port_out (p->port, ch);
+	if ( p->delayBus ) p->hd44780_functions->uPause (p, 1);
+	port_out (p->port + 2, (enableLines|portControl) ^ OUTMASK);
+	if ( p->delayBus ) p->hd44780_functions->uPause (p, 1);
+	port_out (p->port + 2, portControl ^ OUTMASK);
 	sem_signal (semid);
 }
 
-void lcdtime_HD44780_backlight (unsigned char state)
+void lcdtime_HD44780_backlight (PrivateData *p, unsigned char state)
 {
-	backlight_bit = (state?0:SEL);
+	p->backlight_bit = (state?0:SEL);
 
 	// Semaphores not needed because backlight will not go together with
 	// the bacrgraph anyway...
-	port_out (lptPort + 2, backlight_bit ^ OUTMASK);
+	port_out (p->port + 2, p->backlight_bit ^ OUTMASK);
 }
 
-unsigned char lcdtime_HD44780_readkeypad (unsigned int YData)
+unsigned char lcdtime_HD44780_readkeypad (PrivateData *p, unsigned int YData)
 {
 	unsigned char readval;
 
 	sem_wait (semid);
 
-	// 10 bits output or 8 bits if >=3 displays
+	// 10 bits output or 8 bits if >=2 displays
 	// Convert the positive logic to the negative logic on the LPT port
-	port_out (lptPort, ~YData & 0x00FF );
-	if (!extIF) {
-		port_out (lptPort + 2, ( ((~YData & 0x0100) >> 8) | ((~YData & 0x0200) >> 6)) ^ OUTMASK);
+	port_out (p->port, ~YData & 0x00FF );
+	if (p->numDisplays<=2) {
+		// Can't combine >3 displays with >8 keypad output lines
+		port_out (p->port + 2, ( ((~YData & 0x0100) >> 8) | ((~YData & 0x0200) >> 6)) ^ OUTMASK);
 	}
-	if( delayBus ) hd44780_functions->uPause (1);
+	if ( p->delayBus ) p->hd44780_functions->uPause (p, 1);
 
 	// Read inputs
-	readval = ~ port_in (lptPort + 1) ^ INMASK;
+	readval = ~ port_in (p->port + 1) ^ INMASK;
 
 	// Put port back into idle state for backlight
-	port_out (lptPort, backlight_bit ^ OUTMASK);
+	port_out (p->port, p->backlight_bit ^ OUTMASK);
 	sem_signal (semid);
 
 	// And convert value back (MSB first).
@@ -176,5 +177,16 @@ unsigned char lcdtime_HD44780_readkeypad (unsigned int YData)
 		((readval & SELIN) / SELIN <<3) |		/* pin 13 */
 		((readval & PAPEREND) / PAPEREND <<2) |		/* pin 12 */
 		((readval & BUSY) / BUSY <<1) |			/* pin 11 */
-		((readval & ACK) / ACK )) & ~stuckinputs;	/* pin 10 */
+		((readval & ACK) / ACK )) & ~p->stuckinputs;	/* pin 10 */
+}
+
+void lcdtime_HD44780_output(PrivateData *p, int data)
+{
+	// Setup data bus
+	port_out(p->port, data);
+	// Strobe the latch (374 latches on rising edge, 3/574 on trailing.  No matter)
+	port_out (p->port + 2, (LE | p->backlight_bit) ^ OUTMASK);
+        if ( p->delayBus ) p->hd44780_functions->uPause (p, 1);
+	port_out (p->port + 2, (p->backlight_bit) ^ OUTMASK);
+        if ( p->delayBus ) p->hd44780_functions->uPause (p, 1);
 }
