@@ -32,6 +32,11 @@
  * Backlight
  * SEL  (17)	  backlight (optional, see documentation)
  *
+ * Additional output latch strobe
+ * Not part of the hd44780 itself - addl '373/4, '574 chip or similar
+ * (optional, see documentation (eventually))
+ * nLF  (14)	  LE
+ *
  * Keypad connection (optional):
  * Some diodes and resistors are needed, see further documentation.
  * printer port   keypad
@@ -41,11 +46,8 @@
  * D3 (5)	  Y3
  * D4 (6)	  Y4
  * D5 (7)	  Y5
- * D6 (7)	  Y6
- * D7 (7)	  Y7
- * nLF    (14)    Y8
- * INIT   (16)    Y9
- * nSEL   (17)    Y10
+ * D6 (8)	  Y6
+ * D7 (9)	  Y7
  * nACK   (10)    X0
  * BUSY   (11)    X1
  * PAPEREND (12)  X2
@@ -59,11 +61,10 @@
  */
 
 #include "hd44780-winamp.h"
-#include "hd44780.h"
+#include "hd44780-low.h"
 #include "lpt-port.h"
 
 #include "port.h"
-#include "shared/str.h"
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -74,9 +75,10 @@
 // HD44780_senddata
 // HD44780_readkeypad
 
-void lcdwinamp_HD44780_senddata (unsigned char displayID, unsigned char flags, unsigned char ch);
-void lcdwinamp_HD44780_backlight (unsigned char state);
-unsigned char lcdwinamp_HD44780_readkeypad (unsigned int YData);
+void lcdwinamp_HD44780_senddata (PrivateData *p, unsigned char displayID, unsigned char flags, unsigned char ch);
+void lcdwinamp_HD44780_backlight (PrivateData *p, unsigned char state);
+unsigned char lcdwinamp_HD44780_readkeypad (PrivateData *p, unsigned int YData);
+void lcdwinamp_HD44780_output(PrivateData *p, int data);
 
 #define EN1	STRB
 #define EN2	SEL
@@ -84,42 +86,46 @@ unsigned char lcdwinamp_HD44780_readkeypad (unsigned int YData);
 #define RW	LF
 #define RS	INIT
 #define BL	SEL
+#define LE	LF
 
-static unsigned char EnMask[] = { EN1, EN2, EN3 };
-
-static unsigned int lptPort;
-static char stuckinputs = 0;	// if an input line is stuck, it will be ignored
-static char backlight_bit = 0;	// default to low to enable two displays
+static const unsigned char EnMask[] = { EN1, EN2, EN3 };
 
 // initialise the driver
 int
-hd_init_winamp (HD44780_functions * hd44780_functions, lcd_logical_driver * driver, char *args, unsigned int port)
+hd_init_winamp (Driver *drvthis)
 {
+	PrivateData *p = (PrivateData*) drvthis->private_data;
+	HD44780_functions *hd44780_functions = p->hd44780_functions;
+
 	// Reserve the port registers
-	lptPort = port;
-	port_access(lptPort);
-	port_access(lptPort+1);
-	port_access(lptPort+2);
+	port_access_multiple(p->port,3);
 
 	hd44780_functions->senddata = lcdwinamp_HD44780_senddata;
 	hd44780_functions->backlight = lcdwinamp_HD44780_backlight;
 	hd44780_functions->readkeypad = lcdwinamp_HD44780_readkeypad;
 
 	// setup the lcd in 8 bit mode
-	hd44780_functions->senddata (0, RS_INSTR, FUNCSET | IF_8BIT);
-	hd44780_functions->uPause (4100);
-	hd44780_functions->senddata (0, RS_INSTR, FUNCSET | IF_8BIT);
-	hd44780_functions->uPause (100);
-	hd44780_functions->senddata (0, RS_INSTR, FUNCSET | IF_8BIT | TWOLINE | SMALLCHAR);
-	hd44780_functions->uPause (40);
+	hd44780_functions->senddata (p, 0, RS_INSTR, FUNCSET | IF_8BIT);
+	hd44780_functions->uPause (p, 4100);
+	hd44780_functions->senddata (p, 0, RS_INSTR, FUNCSET | IF_8BIT);
+	hd44780_functions->uPause (p, 100);
+
+	common_init (p, IF_8BIT);
+
+	if (p->have_keypad) {
+		// Remember which input lines are stuck
+		p->stuckinputs = lcdwinamp_HD44780_readkeypad (p, 0);
+	}
+
+	// Write new value to output port latches
+	hd44780_functions->output = lcdwinamp_HD44780_output;
 
-	common_init ();
 	return 0;
 }
 
 // lcdwinamp_HD44780_senddata
 void
-lcdwinamp_HD44780_senddata (unsigned char displayID, unsigned char flags, unsigned char ch)
+lcdwinamp_HD44780_senddata (PrivateData *p, unsigned char displayID, unsigned char flags, unsigned char ch)
 {
 	unsigned char enableLines = 0, portControl;
 
@@ -128,25 +134,25 @@ lcdwinamp_HD44780_senddata (unsigned char displayID, unsigned char flags, unsign
 	else
 		portControl = 0;
 
-	portControl |= backlight_bit;
+	portControl |= p->backlight_bit;
 
 	if (displayID == 0)
-		enableLines = EnMask[0] | EnMask[1] | ((extIF) ? EnMask[2] : 0);
+		enableLines = EnMask[0] | EnMask[1] | (p->numDisplays==3 ? EnMask[2] : 0);
 	else
 		enableLines = EnMask[displayID - 1];
 
 	// 40 nS setup time for RS valid to EN high, so set RS
-	port_out (lptPort + 2, portControl ^ OUTMASK);
+	port_out (p->port + 2, portControl ^ OUTMASK);
 
 	// Output the actual data
-	port_out (lptPort, ch);
+	port_out (p->port, ch);
 
-	if( delayBus ) hd44780_functions->uPause (1);
+	if ( p->delayBus ) p->hd44780_functions->uPause (p, 1);
 
 	// then set EN high
-	port_out (lptPort + 2, (enableLines|portControl) ^ OUTMASK);
+	port_out (p->port + 2, (enableLines|portControl) ^ OUTMASK);
 
-	if( delayBus ) hd44780_functions->uPause (1);
+	if ( p->delayBus ) p->hd44780_functions->uPause (p, 1);
 
 	// 80 nS setup from valid data to EN low will be met without any delay
 	// unless you are running a REALLY FAST ISA bus (like 75 MHZ!)
@@ -155,38 +161,46 @@ lcdwinamp_HD44780_senddata (unsigned char displayID, unsigned char flags, unsign
 	// ABOVE TEXT ignored now, using delays if delayBus is specified
 
 	// Set EN low and we're done...
-	port_out (lptPort + 2, portControl ^ OUTMASK);
+	port_out (p->port + 2, portControl ^ OUTMASK);
 
 	// 10 nS data hold time provided by the length of ISA write for EN
 }
 
-void lcdwinamp_HD44780_backlight (unsigned char state)
+void lcdwinamp_HD44780_backlight (PrivateData *p, unsigned char state)
 {
-	backlight_bit = (state?0:nSEL);
+	p->backlight_bit = (state?0:nSEL);
 
-	port_out (lptPort + 2, backlight_bit ^ OUTMASK);
+	port_out (p->port + 2, p->backlight_bit ^ OUTMASK);
 }
 
-unsigned char lcdwinamp_HD44780_readkeypad (unsigned int YData)
+unsigned char lcdwinamp_HD44780_readkeypad (PrivateData *p, unsigned int YData)
 {
 	unsigned char readval;
 
 	// 8 bits output
 	// Convert the positive logic to the negative logic on the LPT port
-	port_out (lptPort, ~YData & 0x00FF );
+	port_out (p->port, ~YData & 0x00FF );
 
-	if( delayBus ) hd44780_functions->uPause (1);
+	if ( p->delayBus ) p->hd44780_functions->uPause (p, 1);
 
 	// Read inputs
-	readval = ~ port_in (lptPort + 1) ^ INMASK;
-
-	// Set output back to idle state for backlight
-	port_out (lptPort + 2, backlight_bit ^ OUTMASK );
+	readval = ~ port_in (p->port + 1) ^ INMASK;
 
 	// And convert value back (MSB first).
 	return (((readval & FAULT) / FAULT <<4) |		/* pin 15 */
 		((readval & SELIN) / SELIN <<3) |		/* pin 13 */
 		((readval & PAPEREND) / PAPEREND <<2) |		/* pin 12 */
 		((readval & BUSY) / BUSY <<1) |			/* pin 11 */
-		((readval & ACK) / ACK )) & ~stuckinputs;	/* pin 10 */
+		((readval & ACK) / ACK )) & ~p->stuckinputs;	/* pin 10 */
+}
+
+void lcdwinamp_HD44780_output(PrivateData *p, int data)
+{
+	// Setup data bus
+	port_out(p->port, data);
+	// Strobe the latch (374 latches on rising edge, 3/574 on trailing.  No matter)
+	port_out (p->port + 2, (LE | p->backlight_bit) ^ OUTMASK);
+        if ( p->delayBus ) p->hd44780_functions->uPause (p, 1);
+	port_out (p->port + 2, (p->backlight_bit) ^ OUTMASK);
+        if ( p->delayBus ) p->hd44780_functions->uPause (p, 1);
 }
