@@ -15,17 +15,17 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 */
-
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+*/
 
 #ifndef LCD_LIRCIN_H
 #define LCD_LIRCIN_H
 
-extern lcd_logical_driver *lircin;
+MODULE_EXPORT int lircin_init (Driver *drvthis);
+MODULE_EXPORT void lircin_close (Driver *drvthis);
+MODULE_EXPORT const char * lircin_get_key (Driver *drvthis);
 
-int lircin_init (struct lcd_logical_driver *driver, char *args);
-
-#define LIRCIN_VERBOSELY 1
+#define LIRCIN_VERBOSELY 0
 
 #define LIRCIN_DEF_PROG "lcdd"
 
