@@ -6,7 +6,6 @@
  * COPYING file distributed with this package.
  *
  * Copyright (c) 1999, William Ferrell, Scott Scriven
- *               2002, Guillaume Filion
  *
  */
 
@@ -15,10 +14,13 @@
 
 #include "screen.h"
 
-extern screen *server_screen;
+extern Screen * server_screen;
+
+extern int rotate_server_screen;
 
 int server_screen_init ();
-int update_server_screen (int timer);
+int server_screen_shutdown ();
+int update_server_screen ();
 int goodbye_screen ();
 
 #endif
